try:
    from xml.etree import cElementTree as etree
except ImportError:
    from xml.etree import ElementTree as etree
import xml.dom.minidom

def prettyPrint(element):
    txt = etree.tostring(element)
    print xml.dom.minidom.parseString(txt).toprettyxml()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
openvas.oaplib - An OAP (OpenVAS Administration Protocol) client interface for Python.

Results from OAP methods
---------------------------

In general all OAPClient methods return either a utf-8 encoded string
or an etree Element. If errors occur an exception subclassed from
`openvas.oaplib.Error` is raised.

Changing the ElementTree library
------------------------------------

``openvas.oaplib`` per default uses xml.etree.ElementTree (resp. it's
C-implementation xml.etree.cElementTree if available). If your
application is using another ElementTree library (e.g. lxml) you can
easily make openvas.oaplib using it by monkey-pathing openvas.oaplib::

  # at the very beginning of your application
  from lxml import etree
  import openvas.oaplib
  openvas.oaplib.etree = etree
  # need to comply to the ElementTree 1.2 interface
  etree.XMLTreeBuilder = etree.XMLParser

You may want to run the test suite to ensure it is really wroking,
though.
"""
#
# Copyright 2010 by Hartmut Goebel <h.goebel@goebel-consult.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

__author__ = "Hartmut Goebel <h.goebel@goebel-consult.de>"
__copyright__ = "Copyright 2010 by Hartmut Goebel <h.goebel@goebel-consult.de>"
__licence__ = "GNU General Public License version 3 (GPL v3)"
__version__ = "0.1.0"

import socket
from OpenSSL import SSL

try:
    from xml.etree import cElementTree as etree
except ImportError:
    from xml.etree import ElementTree as etree


ADMINISTRATOR_ADDRESS = '127.0.0.1'
ADMINISTRATOR_PORT = 9393

# Available Roles
ROLES = ["User", "Admin"]

class Error(Exception):
    """Base class for OAP errors."""
    def __str__(self):
        return repr(self)

class _ErrorResponse(Error):
    def __init__(self, cmd, *args):
        if cmd.endswith('_response'):
            cmd = cmd[:-9]
        super(_ErrorResponse, self).__init__(cmd, *args)
        
    def __str__(self):
        return '%s %s' % self.args[1:3]

class ClientError(_ErrorResponse):
    """command issued could not be executed due to error made by the client"""
    
class ServerError(_ErrorResponse):
    """error occurred in the administrator during the processing of this command"""
    
class ResultError(Error):
    """Get invalid answer from Server"""
    def __str__(self):
        return 'Result Error: answer from command %s is invalid' % self.args

class AuthFailedError(Error):
    """Authentication failed."""


class _SSLSocket(object):
    """A Socket wrapped by a SSL connection."""
    __credits__ = "Hartmut Goebel, Tryton"
    
    def __init__(self, socket):
        # accept SSLv3 and TLS1. Disable SSLv2 since it is insecure.
        # NB: SSLv23_METHOD includes TLS1
        ctx = SSL.Context(SSL.SSLv23_METHOD)
        ctx.set_options(SSL.OP_NO_SSLv2)
        self.socket = SSL.Connection(ctx, socket)

    def recv(self, size):
        while 1:
            try:
                return self.socket.recv(size)
            except SSL.ZeroReturnError, e:
                return ''
            except (SSL.WantWriteError, SSL.WantReadError), e:
                pass

    def __getattr__(self, name):
        return getattr(self.socket, name)

def XMLNode(tag, *kids, **attrs):
    n = etree.Element(tag, attrs)
    for k in kids:
        if isinstance(k, basestring):
            assert n.text is None
            n.text = k
        else:
            n.append(k)
    return n


class OAPClient(object):
    def __init__(self, host, port=ADMINISTRATOR_PORT, username=None, password=None):
        self.socket = None
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.session = None

    def open(self, username=None, password=None):
        """Open a connection to the administrator and authenticate the user."""
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket = sock = _SSLSocket(sock)
        # do not set timeout, OpenSSL does not like it
        sock.connect((self.host, self.port))
        sock.set_connect_state()
        try:
            sock.do_handshake()
        except SSL.SysCallError:
            raise RuntimeError, "socket connection broken"
        self.authenticate(username, password)

    def close(self):
        """Close the connection to the administrator"""
        self.socket.shutdown()
        self.socket = None

    def _send(self, data):
        """Send OAP data to the administrator and read the result.

        `data` may be either an unicode string, an utf-8 encoded
        string or an etree Element. The result is as an etree Element.
        """
        BLOCK_SIZE = 1024
        if etree.iselement(data):
            #print '>>>', etree.tostring(data)
            root = etree.ElementTree(data)
            root.write(self.socket, 'utf-8')
        else:
            if isinstance(data, unicode):
                data = data.encode('utf-8')
            self.socket.send(data)
        parser = etree.XMLTreeBuilder()
        while 1:
            res = self.socket.recv(BLOCK_SIZE)
            #print repr(res)
            parser.feed(res)
            if len(res) < BLOCK_SIZE:
                break
        root = parser.close()
        #print '<<<', etree.tostring(root)
        return root

    def _check_response(self, response):
        """Check the response read from the administrator.

        If the response status is 4xx a ClientError is raised, if the
        status is 5xx a ServerError is raised.
        """
        status = response.get('status')
        if status is None:
            raise RunTimeError('response is missing status: %s'
                               % etree.tostring(response))
        if status.startswith('4'):
            raise ClientError(response.tag, status,
                              response.get('status_text'))
        elif status.startswith('5'):
            raise ServerError(response.tag, status,
                              response.get('status_text'))
        return status

    def _text_command(self, request):
        response = self._send(request)
        self._check_response(response)
        return response.text

    def _xml_command(self, request):
        response = self._send(request)
        self._check_response(response)
        return response


    def __generic_get(self, type, name, element2dict):
        """
        Generic function for retrieving information using <get_XXX>.

        If 'name' is not given, retrieve all thingies. Result is a
        generator of dicts with the thingies datas.
        
        If 'name' is set, get only the thingy identified by its name.
        Result is a single dict with the thingies data.
        """
        request = XMLNode('get_%s' % type)
        if name is not None:
            request.set('name', name)
        response = self._xml_command(request)
        if name:
            return element2dict(response[0])
        else:
            return (element2dict(elem) for elem in response)


    # --- basic commands ---

    def authenticate(self, username=None, password=None):
        """
        Authenticate a user to the administrator.

        If username or password are not gives, the values passed when
        instanciating the object are used.

        Raises AuthFailedError if authentication failed due to wrong
        credentials.
        """
        if username is None:
            username = self.username
        if password is None:
            password = self.password
        request = XMLNode("authenticate",
                          XMLNode("credentials",
                                  XMLNode("username", username),
                                  XMLNode("password", password),
                                  ))
        try:
            self._text_command(request)
            # if not status: connection closed, raise error
        except ClientError:
            raise AuthFailedError(username)

    def xml(self, xmldata, xml_result=False):
        """Low-level interface to send OAP XML to the administrator.

        `xmldata` may be either an unicode string, an utf-8 encoded
        string or an etree Element. If `xml_result` is true, the
        result is returned as an etree Element, otherwise a unicode
        string is returned.

        Please see the modules documentation about how to change the
        ElementTree library used.
        """
        if xml_result:
            return self._xml_command(xmldata)
        else:
            return self._text_command(xmldata)

    def help(self):
        """
        Retrieve help from OAP server.
        """
        return self._text_command(XMLNode('help'))


    def get_version(self):
        """
        Retrieve protocol versions supported by the administrator.

        Returns two values: first is the preferred version, second
        element is a list of all versions (including the preferred
        one). Versions are strings, the order is as returned from the
        administrator.
        """
        response = self._xml_command(XMLNode('get_version'))
        preferred = None
        versions = []
        for ver in response.findall('version'):
            versions.append(ver.text)
            if ver.get('preferred'):
                if preferred:
                    raise ServerError("OAP command 'get_version' returned "
                                      "several preferred versions")
                preferred = ver.text
        if not preferred:
            raise ServerError("OAP command 'get_version' did not return "
                              "a preferred version")
        return preferred, versions

    # --- users ---

    def create_user(self, name, password, role, hosts):
        """
        Create a new user
        """
        request = XMLNode('create_user',
                          XMLNode('name', name),
                          XMLNode('password', password),
                          XMLNode('role', role),
                          XMLNode('hosts', hosts),
                          )
        self._text_command(request)

    def modify_user(self, name, password=None, role=None, hosts=None):
        """
        Modify an existing user
        """
        request = XMLNode('modify_user',
                          XMLNode('name', name)
                          )
        if password:
            request.append(XMLNode('password', password, modify="1"))
        else:
            request.append(XMLNode('password', modify="0"))
        if role:
            request.append(XMLNode('role', role))
        if hosts:
            request.append(XMLNode('hosts', hosts))
        self._text_command(request)


    def get_users(self, name=None):
        """
        Get information about user(s).

        If 'name' is not given, retrieve all users. Result is a
        generator of dicts with the users datas.
        
        If 'name' is set, get only the users identified by its name.
        Result is a single dict with the users data.
        """

        def user2dict(element):
            entry = dict((elem.tag, elem.text) for elem in element)
            # todo: find a better solution for this
            entry['hosts_allow'] = element.find('hosts').get('allow')
            # work around asymetry in openvas-administrator
            if entry['role'] == "Administrator":
                entry['role'] = "Admin"
            return entry

        return self.__generic_get('users', name, user2dict)


    def delete_user(self, name):
        """
        Delete an user identified by his name
        """
        request = XMLNode('delete_user', name=name)
        return self._text_command(request)


    # --- configs ---

    def describe_feed(self):
        """
        Get details of the NVT feed this administrator uses.

        Returns a dict containing the information.
        """

        def feed2dict(element):
            # todo: currently_syncing
            entry = dict((elem.tag, elem.text) for elem in element)
            syncing = element.find('currently_syncing')
            if syncing is not None:
                entry['currently_syncing'] = feed2dict(syncing)
            return entry

        request = XMLNode('describe_feed')
        res = self._xml_command(request)
        return feed2dict(res.find('feed'))

    def sync_feed(self):
        """
        Synchronize with an NVT feed
        """
        request = XMLNode('sync_feed')
        return self._text_command(request)


    def describe_auth(self):
        """
        Get details about the used authentication methods
        """
        request = XMLNode('describe_auth')
        return self._text_command(request).decode('base64')


    def get_settings(self, name=None):
        """
        Get information about settings
        """
        request = XMLNode('get_settings')
        res = self._xml_command(request)
        settings = []
        for scanner_settings in res.findall('scanner_settings'):
            # copy attributes
            scan_setting = dict(scanner_settings.items())
            setting_items = scan_setting['items'] = {}
            for setting in scanner_settings.findall('setting'):
                setting_items[setting.get('name')] = setting.text
            settings.append(scan_setting)
        return settings



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010 by Hartmut Goebel <h.goebel@goebel-consult.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

import sys

if sys.version_info < (2,5) or sys.version_info >= (3,):
    sys.exit('openvas.oaplib requires Python >= 2.5 but < 3.0')

from setuptools import setup, find_packages
import os

long_description="\n\n".join([
    open(os.path.join("README.txt")).read(),
    ])

setup(
    name = "openvas.oaplib",
    version = "0.0.1",
    install_requires = ['setuptools', 'argparse'],

    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['openvas'],

    package_data = {
        # If any package contains *.txt or *.rst files, include them:
        '': ['*.txt', '*.rst'],
        },

    # metadata for upload to PyPI
    author = "Hartmut Goebel",
    author_email = "h.goebel@goebel-consult.de",
    description = "An OAP (OpenVAS Administration Protocol) client interface for Python",
    long_description = long_description,
    license = "GPL 3.0",
    keywords = "OpenVAS,OAP,OpenVAS Administration Protocol,Nessus",
    url          = "http://www.openvas.org/",
    #download_url = "http://pdfposter.origo.ethz.ch/download",
    classifiers = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'Intended Audience :: Information Technology',
    'Intended Audience :: System Administrators',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Natural Language :: English',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.5',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Topic :: Security',
    'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    # these are for easy_install (used by bdist_*)
    zip_safe = True,
    entry_points = {
        "console_scripts": [
            "oap-cli = openvas.oaplib.cmd:run",
        ],
    },
)



# -*- coding: utf-8 -*-
"""
Testsuite for openvas.oaplib
"""
#
# Copyright 2010 by Hartmut Goebel <h.goebel@goebel-consult.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

__author__ = "Hartmut Goebel <h.goebel@goebel-consult.de>"
__copyright__ = "Copyright 2010 by Hartmut Goebel <h.goebel@goebel-consult.de>"
__licence__ = "GNU General Public License version 3 (GPL v3)"

import unittest
import os
import inspect
import random
import socket
import sys

import openvas.oaplib
from openvas.oaplib import etree

class OAPTestBase(unittest.TestCase):
    _OAP_HOST = None
    _OAP_PORT = None
    _OAP_USER = None
    _OAP_PASSWD = None

    def setUp(self):
        self.client = openvas.oaplib.OAPClient(self._OAP_HOST, self._OAP_PORT,
                                               self._OAP_USER, self._OAP_PASSWD)
        self.client.open()

    def tearDown(self):
        self.client.close()


class TestOAP(OAPTestBase):

    def test_help(self):
        self.client.help()

    def test_xml_0(self):
        text = self.client.xml('<help/>')
        self.assert_(isinstance(text, basestring))
        self.assert_(text != '')

    def test_xml_1(self):
        xml = self.client.xml('<help/>', xml_result=True)
        self.assert_(etree.iselement(xml))

    def test_get_version(self):
        ver = self.client.get_version()
        self.assert_(isinstance(ver, tuple))
        self.assertEqual(len(ver), 2)
        self.assert_(isinstance(ver[0], basestring))
        self.assert_(isinstance(ver[1], (tuple, list)))

    #--- users ---

    def test_user(self):
        name = "oap_test_user"
        password = ''.join((name, str(id(self))))
        try:
            self.client.create_user(name, password, "User", "allow any")
            users = self.client.get_users()
            self.assert_(inspect.isgenerator(users))
            for user in users:
                self.assert_(isinstance(user, dict))
            user = self.client.get_users(name)
            self.assert_(isinstance(user, dict))
            self.assert_(user['name'] == name)
        finally:
            self.client.delete_user(name)

    def test_modify_user(self):
        name = "oap_test_user"
        password = ''.join((name, str(id(self))))
        try:
            self.client.create_user(name, password, "User", "allow all")
            self.client.modify_user(name, role="Admin")
            user = self.client.get_users(name)
            self.assert_(user['role'] == "Admin")

            self.client.modify_user(name, hosts="deny all")
            user = self.client.get_users(name)
            self.assert_(user['hosts'] == "deny all")

            self.client.modify_user(name, password=password[::-1])
        finally:
            self.client.delete_user(name)

    #----

    def test_describe_feed(self):
        feed = self.client.describe_feed()
        self.assert_(isinstance(feed, dict))
        self.assert_(feed.has_key('name'))
        self.assert_(feed.has_key('description'))

    def test_sync_feed(self):
        feed = self.client.sync_feed()
        self.assert_(feed is None)
        feed = self.client.describe_feed()
        # assume our request is faster than the sync finishes
        self.assert_(feed.has_key('currently_syncing'))

    #----
    
    def test_describe_auth(self):
        try:
            text = self.client.describe_auth()
        except openvas.oaplib.ClientError, e:
            self.assertEqual(e.args,
                             ('describe_auth', '404', 'Resource missing'))
        else:
            self.assert_(isinstance(text, basestring))

    def test_get_settings(self):
        settings = self.client.get_settings()
        self.assert_(isinstance(settings, list))
        for setting in settings:
            self.assert_(setting.has_key('sourcefile'))
            self.assert_(setting.has_key('editable'))
            self.assert_(setting.has_key('items'))
            self.assert_(isinstance(setting['items'], dict))


class main(unittest.main):
    def parseArgs(self, argv):
        """

Examples:
  %prog                            - run default set of tests
  %prog MyTestSuite                - run suite 'MyTestSuite'
  %prog MyTestCase.testSomething   - run MyTestCase.testSomething
  %prog MyTestCase                 - run all 'test*' test methods
                                               in MyTestCase
        """
        import getpass
        import optparse
        parser = optparse.OptionParser('usage: %prog [options] [test] [...]' +
                                       self.parseArgs.__doc__.rstrip(),
                                       add_help_option=False)
        parser.add_option("--help", action="help",
                          help="show this help message and exit")
        parser.set_defaults(verbosity=1)
        parser.add_option('-v', '--verbose', dest='verbosity',
                          action='store_const', const=2)
        parser.add_option('-q', '--quiet', dest='verbosity',
                          action='store_const', const=0)

        group = parser.add_option_group('Options for connecting to the '
                                        'administrator')
        group.add_option('-h', '--host',
                          default=openvas.oaplib.ADMINISTRATOR_ADDRESS,
                          help=("Connect to administrator on host HOST "
                                "(default: %default)"))
        group.add_option('-p', '--port', type=int,
                          default=openvas.oaplib.ADMINISTRATOR_PORT,
                          help="Use port number PORT (default: %default)")
        group.add_option('-u', '--username', default=getpass.getuser(),
                          help="OAP username (default: current user's name)")
        group.add_option('-w', '--password',
                          help=("OAP password (use '-' to be prompted, "
                                "default: same as username)"))

        opts, args = parser.parse_args(argv[1:])

        if not (0 < opts.port < 65536):
            parser.error('Administrator port must be a number between 1 and 65535.')
        if opts.password is None:
            opts.password = opts.username
        elif opts.password == '-':
            opts.password = getpass.getpass('Enter password for openvas-administrator: ')

        self.verbosity = int(opts.verbosity)
        OAPTestBase._OAP_HOST = opts.host
        OAPTestBase._OAP_PORT = opts.port
        OAPTestBase._OAP_USER = opts.username
        OAPTestBase._OAP_PASSWD = opts.password
        
        if len(args) == 0 and self.defaultTest is None:
            self.test = self.testLoader.loadTestsFromModule(self.module)
            return
        if len(args) > 0:
            self.testNames = args
        else:
            self.testNames = (self.defaultTest,)
        self.createTests()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
openvas.oaplib.cmd - An OAP (OpenVAS Administration Protocol) command line client.
"""
#
# Copyright 2010 by Hartmut Goebel <h.goebel@goebel-consult.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

__author__ = "Hartmut Goebel <h.goebel@goebel-consult.de>"
__copyright__ = "Copyright 2010 by Hartmut Goebel <h.goebel@goebel-consult.de>"
__licence__ = "GNU General Public License version 3 (GPL v3)"
__version__ = '0.0.1'

from openvas.oaplib import *
# use the same etree implementations as oaplib
from openvas.oaplib import etree

import inspect
import sys
import time
import argparse
import textwrap
import getpass

try:
    from gettext import gettext
except ImportError:
    def gettext(message):
        return message
_ = gettext


class OAPCommandClient(object):
    def __init__(self, args):
        self.client = OAPClient(args.host, args.port,
                                args.username, args.password)

    def run_command(self, optparser, args):
        mod = inspect.getmodule(self)
        cmd = getattr(mod, args.command_name.replace('-', '_'))
        self.client.open()
        try:
            res = cmd.run(self.client, args)
        except Error, e:
            sys.exit('Error: %s' % e)
        finally:
            self.client.close()
        return res
        
    @classmethod
    def _find_commands(klass):
        def is_OAPCommand(c):
            return inspect.isclass(c) and OAPCommand in c.__bases__
        
        classes = inspect.getmembers(inspect.getmodule(klass), is_OAPCommand)
        return (c[1] for c in classes)


class OAPCommand(object):
    @classmethod
    def _argparse(klass, subparsers):
        cmd_name = klass.__name__.replace('_', '-')
        description = klass.__doc__
        help = description.lstrip().split('\n', 1)[0]
        description = textwrap.dedent(description)
        sp = subparsers.add_parser(cmd_name,
                                   formatter_class=argparse.RawDescriptionHelpFormatter,
                                   help=help,
                                   description=description)
        klass._add_arguments(sp)

#--- basic commands ---

class help(OAPCommand):
    """
    Get help message from OAP server
    """
    
    @staticmethod
    def _add_arguments(sp): pass

    @staticmethod
    def run(oap_client, args):
        res = oap_client.help()
        print res.strip('\n')


class xml(OAPCommand):
    """
    Send plain XML command
    """
    
    @staticmethod
    def _add_arguments(sp):
        sp.add_argument('xml', nargs=1, help='XML text/document to send')

    @staticmethod
    def run(oap_client, args):
        res = oap_client.xml(args.xml[0], xml_result=True)
        print etree.tostring(res)


class get_version(OAPCommand):
    """
    Get the OAP versions supported by the administrator.

    Versions are separated by spaces. The first version is the one the
    administrator prefers.
    """
    
    @staticmethod
    def _add_arguments(sp): pass

    @staticmethod
    def run(oap_client, args):
        pref, versions = oap_client.get_version()
        print pref, ' '.join(versions)


#--- user management ---

class create_user(OAPCommand):
    """
    Create a new user
    """
    
    @staticmethod
    def _add_arguments(sp):
        sp.add_argument('name', help="Name for the new user")
        sp.add_argument('--new_password', required=True,
                        help="Password for new user ('-' to be prompted)")
        sp.add_argument('--role', help="Role user", choices=ROLES)

    @staticmethod
    def run(oap_client, args):
        password = args.new_password
        if password == '-':
            password = getpass.getpass('Password for new OpenVAS user: ')
        # todo: read hosts from stdin
        hosts = ""
        oap_client.create_user(name=args.name, password=password,
                               role=args.role, hosts=hosts)

class get_users(OAPCommand):
    """
    Get details about one or all users
    """
    
    @staticmethod
    def _add_arguments(sp):
        sp.add_argument('name', nargs='*',
                        help='Name of user about which to retrieve details')

    @staticmethod
    def run(oap_client, args):
        def print_user(user):
            print "%(name)-10s\t%(role)-14s\t" % user,
            # todo: complete this depending on allow/denny state
            if user['hosts'] is not None:
                print user['hosts'].replace('\n',' ')
            else:
                print

        if args.name:
            for name in args.name:
                print_user(oap_client.get_users(name))
        else:
            for user in oap_client.get_users():
                print_user(user)


class modify_user(OAPCommand):
    """
    Modfiy an existing user
    """
    
    @staticmethod
    def _add_arguments(sp):
        sp.add_argument('name', help="Name for the new user")
        sp.add_argument('--new-password',
                        help="Password for new user ('-' to be prompted)")
        sp.add_argument('--role', help="Role user", choices=ROLES)

    @staticmethod
    def run(oap_client, args):
        password = args.new_password
        if password == '-':
            password = getpass.getpass('Password for new OpenVAS user: ')
        # todo: read hosts from stdin
        hosts = ""
        oap_client.modify_user(name=args.name, password=password,
                               role=args.role, hosts=hosts)
    

class delete_user(OAPCommand):
    """
    Delete one or more users
    """
    
    @staticmethod
    def _add_arguments(sp):
        sp.add_argument('name', nargs='+', help="Name of user to delete")

    @staticmethod
    def run(oap_client, args):
        for name in args.name:
            oap_client.delete_user(name)

#--- feeds ---

class describe_feed(OAPCommand):
    """
    Get details about the NVT feed this administrator uses
    """
    
    @staticmethod
    def _add_arguments(sp): pass

    @staticmethod
    def run(oap_client, args):
        res = oap_client.describe_feed()
        print 'Name:\t\t%s' % res['name'].strip()
        if res.has_key('version'):
            print 'Version:\t%s' % res['version'].strip()
        print textwrap.fill(res['description'],
                            initial_indent   ='Description:\t',
                            subsequent_indent='\t\t')
        syncing = res.get('currently_syncing', None)
        if syncing:
            print 'Currently Synchronizing:\t%s\t%s' % (syncing['user'].strip(),
                                        syncing['timestamp'])

class sync_feed(OAPCommand):
    """
    Synchronize with the NVT feed
    """
    
    @staticmethod
    def _add_arguments(sp): pass

    @staticmethod
    def run(oap_client, args):
        oap_client.sync_feed()


#--- misc. ---

class describe_auth(OAPCommand):
    """
    Get details about the used authentication methods
    """
    @staticmethod
    def _add_arguments(sp): pass

    @staticmethod
    def run(oap_client, args):
        print oap_client.describe_auth()


class get_settings(OAPCommand):
    """
    Get settings
    """
    
    @staticmethod
    def _add_arguments(sp): pass
        
    @staticmethod
    def run(oap_client, args):
        settings = oap_client.get_settings()
        first = True
        for setting in settings:
            print ('Setting: %s\t(%seditable)'
                   % (setting['sourcefile'],
                      '' if int(setting['editable']) else 'not '))
            for k, v in sorted(setting['items'].items()):
                print '  %s\t%s' % (k, v)
            if not first:
                print
            first = False


def run():
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument("--help", action="help",
                        help=_("show this help message and exit"))
    parser.add_argument("--version", action='version', version=__version__)
    parser.add_argument('-v', '--verbose', action='store_true',
                        help="Verbose messages.")
    parser.add_argument('-P', '--prompt', action='store_true',
                        help="Prompt to exit.")

    group = parser.add_argument_group('Options for connecting to the administrator')
    group.add_argument('-h', '--host', default=ADMINISTRATOR_ADDRESS,
                       help=("Connect to administrator on host HOST "
                             "(default: %(default)s)"))
    group.add_argument('-p', '--port', type=int, default=ADMINISTRATOR_PORT,
                       help="Use port number PORT (default: %(default)s)")
    group.add_argument('-u', '--username', default=getpass.getuser(),
                       help="OAP username (default: current user's name)")
    group.add_argument('-w', '--password',
                       help="OAP password (default: same as username)")

    subparsers = parser.add_subparsers(dest='command_name',
                                       title='Available OAP commands',
                                       help='Get help for sub-commands with <cmd> --help')

    for cmd in OAPCommandClient._find_commands():
        cmd._argparse(subparsers)

    args = parser.parse_args()
    
    if not (0 < args.port < 65536):
        parser.error('Administrator port must be a number between 1 and 65535.')
    if args.password is None:
        args.password = args.username

    client = OAPCommandClient(args)
    client.run_command(parser, args)

if __name__ == '__main__':
    run()
